<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://welaunch.io/plugins/woocommerce-ultimate-pricing/
 * @since      1.0.0
 *
 * @package    WooCommerce_delivery
 * @subpackage WooCommerce_delivery/public
 */
class WooCommerce_Ultimate_Pricing_Tiered_Pricing extends WooCommerce_Ultimate_Pricing {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	protected $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of this plugin.
	 */
	protected $version;

	/**
	 * options of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      array    $options
	 */
	protected $options;


	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) 
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}
	
	public function init()
	{
		global $woocommerce_ultimate_pricing_options, $tieredPricingDiscountsApplied, $tieredPriceApplied;
		$this->options = $woocommerce_ultimate_pricing_options;

		if (!$this->get_option('enable')) {
			return false;
		}

		if (!$this->get_option('tieredPricingEnabled')) {
			return false;
		}

		if(is_admin()) { 
			return false;
		}

		$this->user_id = get_current_user_id();

		add_action($this->get_option('tieredPricingPosition'), array($this, 'show_table'), $this->get_option('tieredPricingPriority'));
		add_shortcode('woocommerce_ultimate_pricing_tiered_pricing_table', array($this, 'show_table_shortcode'));


		add_action( 'woocommerce_before_calculate_totals', array($this, 'change_tiered_price'), 20, 1 );
		add_filter( 'woocommerce_cart_item_price', array($this, 'change_tiered_price_in_cart'), 15, 3);

	}

	public function check_tiered_pricing_discounts($product) 
	{
		global $tieredPricingDiscountsApplied, $tieredPriceApplied;

		if(!$product) {
			return false;
		}

		$productId = $product->get_id();
		$variationId = $productId;
		if($product->is_type('variation')) {
			$product = wc_get_product($product->get_parent_id());
			$productId = $product->get_id();
		}

		if(!isset($tieredPricingDiscountsApplied[$variationId])) {
			$tieredPricingDiscountsApplied[$variationId] = array(
				'applied' => false,
				'discounts' => array(),
				'type' => '',
			);	
		}

		// Already applied rules - do not apply again performance
		// if(isset($tieredPricingDiscountsApplied[$productId]['applied']) && $tieredPricingDiscountsApplied[$productId]['applied'] && ) {
		// 	return $tieredPricingDiscountsApplied[$productId]['newPrice'];
		// }

		// Custom Tiered Pricing
		$customTieredPricing = get_post_meta($productId, 'woocommerce_ultimate_pricing_tiered_pricing', true);
		if(!empty($customTieredPricing)) {
			if(isset($customTieredPricing['type']) && isset($customTieredPricing['discounts']) && !empty($customTieredPricing['discounts'])) {
				$tieredPricingDiscountsApplied[$variationId]['applied'] = true;
			    $tieredPricingDiscountsApplied[$variationId]['type'] = $customTieredPricing['type'];
		    	$tieredPricingDiscountsApplied[$variationId]['discounts'] = array_filter( $customTieredPricing['discounts'] );
		    	return true;
			}
		}

		$productIdToCheckTaxonomies = $productId;
		if($product->is_type('variation')) {
			$productIdToCheckTaxonomies = $product->get_parent_id();
		}
		
		$role = $this->get_user_role($this->user_id);
		$B2BKingCustomergroup = get_user_meta($this->user_id, 'b2bking_customergroup', true);
		
		$tieredPricingCount = $this->get_option('tieredPricingCount');
		for ($i=1; $i <= $tieredPricingCount; $i++) { 

			if(!$this->get_option('tieredPricingRule' . $i . 'Enabled')) {
				continue;
			}

		    $roles = $this->get_option('tieredPricingRule' . $i . 'Roles');
		    if(!empty($roles)) {
			   		
			   	// No Role or Guest
			   	if(!$role) {
			   		continue;
			   	}

				if(!in_array($role, $roles)) {
					continue;
				}
		    }

		    $B2BKingCustomerGroups = $this->get_option('tieredPricingRule' . $i . 'B2BKingCustomerGroups');
		    if(!empty($B2BKingCustomerGroups)) {

			   	if(!in_array($B2BKingCustomergroup, $B2BKingCustomerGroups)) {
			   		continue;
		   		}
		    }

		    $dateRange = $this->get_option('tieredPricingRule' . $i . 'DateRange');
		    if(!empty($dateRange)) {
		    	$dateRange = explode(' to ', $dateRange);
		    	$startDate = $dateRange[0];
		    	$endDate = $dateRange[1];
				if(!empty($startDate) && !empty($endDate)) {
					$currentDateObject = new DateTime();
					$startDateObject = new DateTime($startDate);
					$endDateObject = new DateTime($endDate . ' 23:59');
				    if($currentDateObject < $startDateObject || $currentDateObject > $endDateObject) {
				        continue;
				    }
				}
		    }

		    $categories = $this->get_option('tieredPricingRule' . $i . 'Categories');
		    if(!empty($categories)) {
			    if( !has_term($categories, 'product_cat', $productIdToCheckTaxonomies) ) {
			        continue;
			    }
		    }

		    $products = $this->get_option('tieredPricingRule' . $i . 'Products');
		    if(!empty($products)) {
			    if( !in_array($productIdToCheckTaxonomies, $products)) {
			        continue;
			    }
		    }

		    $tags = $this->get_option('tieredPricingRule' . $i . 'Tags');
		    if(!empty($tags)) {
			    if( !has_term($tags, 'product_tag', $productIdToCheckTaxonomies) ) {
			        continue;
			    }
		    }

		    $reviews = $this->get_option('tieredPricingRule' . $i . 'Reviews');
		    if(!empty($reviews)) {
			    $userReviews = $this->get_user_review_count($this->user_id);
			    if($userReviews < $reviews) {
			    	continue;
			    }
		    }

		    $orders = $this->get_option('tieredPricingRule' . $i . 'Orders');
		    if(!empty($orders)) {
			    $userOrders = $this->get_user_order_count($this->user_id);
			    if($userOrders < $orders) {
			    	continue;
			    }
		    }


			$tieredPricingDiscountsApplied[$variationId]['applied'] = true;
		    $tieredPricingDiscountsApplied[$variationId]['type'] = $this->get_option('tieredPricingRule' . $i . 'Type');
	    	$tieredPricingDiscountsApplied[$variationId]['discounts'] = array_filter( $this->get_option('tieredPricingRule' . $i . 'Discounts') );
		}

		return true;
	}

	public function show_table_shortcode($atts = array())
	{
		ob_start();
		
		$this->show_table($atts);

        $output_string = ob_get_contents();
        ob_end_clean();	 

        return $output_string;   

	}


	public function show_table($atts = array())
	{
    	global $product, $tieredPricingDiscountsApplied, $tieredPriceApplied;

    	$original_product = $product;

        extract(shortcode_atts(array(
            'product_id' => '',
        ), $atts));

        if(!empty($product_id)) {
    		$product = wc_get_product($product_id);
        }
        
        if(!$product) {
        	return esc_html__('No Product found', 'woocommerce-ultimate-pricing');
        }

        $productId = $product->get_id();

		$this->check_tiered_pricing_discounts($product);
		
		if( !isset($tieredPricingDiscountsApplied[$productId]) || !$tieredPricingDiscountsApplied[$productId]['applied']) {
			return;
		}

		?>

		<style>
			.woocommerce-ultimate-pricing-tiered-pricing-table-row-active, .woocommerce-ultimate-pricing-tiered-pricing-table-row-active td, .woocommerce-ultimate-pricing-tiered-pricing-table-row-active span.amount,
			.woocommerce-ultimate-pricing-tiered-pricing-table tbody tr:hover, .woocommerce-ultimate-pricing-tiered-pricing-table tbody tr:hover td, .woocommerce-ultimate-pricing-tiered-pricing-table tbody tr:hover span.amount {
				background-color: <?php echo $this->get_option('tieredPricingActiceRowBackgroundColor') ?> ;
				color: <?php echo $this->get_option('tieredPricingActiceRowColor') ?> ;
				cursor: pointer;
			}
		</style>

		<table class="woocommerce-ultimate-pricing-tiered-pricing-table" data-type="<?php echo $tieredPricingDiscountsApplied[$productId]['type'] ?>">
			<thead>
				<tr>
					<th><?php esc_html_e($this->get_option('tieredPricingTextsQuantity')) ?></th>
					<th><?php esc_html_e($this->get_option('tieredPricingTextsDiscount')) ?></th>
					<th><?php esc_html_e($this->get_option('tieredPricingTextsPrice')) ?></th>
				</tr>
			</thead>
			<tbody>
				<?php 
				$productPrice = $product->get_price();
				foreach ($tieredPricingDiscountsApplied[$productId]['discounts'] as $tieredPricingDiscount) {

					$tieredPricingDiscount = explode('|', $tieredPricingDiscount);
					if(!isset($tieredPricingDiscount[0]) || !isset($tieredPricingDiscount[1])) {
						continue;
					}

					$tieredPricingDiscountQuantity = $tieredPricingDiscount[0];
					$tieredPricingDiscountDiscount = $tieredPricingDiscount[1];

					if($tieredPricingDiscountsApplied[$productId]['type'] == "percentage") {
						$tieredPricingDiscountDiscountHTML = $tieredPricingDiscountDiscount . '%';
						$tieredPricingPrice = $productPrice - ( ($tieredPricingDiscountDiscount / 100) * $productPrice);
					} elseif($tieredPricingDiscountsApplied[$productId]['type'] == "fixed_total") {
						$tieredPricingDiscountDiscountHTML = wc_price($tieredPricingDiscountDiscount);
						$tieredPricingPrice = $tieredPricingDiscountDiscount;
					} else {
						$tieredPricingDiscountDiscountHTML = wc_price($tieredPricingDiscountDiscount);
						$tieredPricingPrice = $productPrice - $tieredPricingDiscountDiscount;
					}

					$tieredPricingDiscountNext = explode( '|', next($tieredPricingDiscountsApplied[$productId]['discounts']) );
					if(!isset($tieredPricingDiscountNext[0]) || !isset($tieredPricingDiscountNext[1])) {
						$tieredPricingDiscountQuantityNext = 999999999;
						$tieredPricingDiscountQuantityNextHTML = '+';
					} else {
						$tieredPricingDiscountQuantityNext = $tieredPricingDiscountNext[0] - 1;
						$tieredPricingDiscountQuantityNextHTML = ' – ' . $tieredPricingDiscountQuantityNext;
					}

					if($tieredPricingPrice < 0) {
						$tieredPricingPrice = 0;
					}
					
					$tieredPricingPrice = number_format($tieredPricingPrice, get_option('woocommerce_price_num_decimals'));

					?>

					<tr data-min-quantity="<?php echo $tieredPricingDiscountQuantity ?>" data-max-quantity="<?php echo $tieredPricingDiscountQuantityNext ?>" data-discount="<?php echo $tieredPricingDiscountDiscount ?>" data-price="<?php echo $tieredPricingPrice ?>">
						<td><?php echo $tieredPricingDiscountQuantity . $tieredPricingDiscountQuantityNextHTML ?></td>
						<td><?php echo $tieredPricingDiscountDiscountHTML ?></td>
						<?php if($product->is_type('variable')) { ?>
						<td><?php echo esc_html__('From ', 'woocommerce-ultimate-pricing') . wc_price( $tieredPricingPrice ) ?></td>	
						<?php } else { ?>
						<td><?php echo wc_price( $tieredPricingPrice ) ?></td>	
						<?php } ?>
					</tr>

					<?php
					$tieredPricingDiscountQuantityBefore = $tieredPricingDiscountQuantity;
				}
				?>
			</tbody>
		</table>

		<?php
		$product = $original_product;
	}

	
	public function change_tiered_price($cart) 
	{
		global $product, $tieredPricingDiscountsApplied, $tieredPriceApplied;

		if ( is_admin() && ! defined( 'DOING_AJAX' ) ) {
			return;
		}

		if(!is_cart() && !is_checkout()) {
			return;
		}

		$cartItems = $cart->get_cart();

		// Iterate through each cart item
		foreach($cartItems as $cartItemKey => $cartItemValue ) {

			$cartItemProduct = $cartItemValue['data'];
			if(!$cartItemProduct) {
				continue;
			}

			$productId = $cartItemProduct->get_id();
			$variationId = $productId;
			if($cartItemProduct->is_type('variation')) {
				$productId = $cartItemProduct->get_parent_id();
			}

			if(is_array($tieredPriceApplied) && array_key_exists($variationId, $tieredPriceApplied)) {
				continue;
			}

			$this->check_tiered_pricing_discounts($cartItemProduct);

			$cartItemQuantity = $cartItemValue['quantity'];			
			$productPrice = $cartItemProduct->get_price();

			if( !isset($tieredPricingDiscountsApplied[$variationId]) || !$tieredPricingDiscountsApplied[$variationId]['applied']) {
				continue;
			}

			foreach ($tieredPricingDiscountsApplied[$variationId]['discounts'] as $tieredPricingDiscount) {

				$tieredPricingDiscount = explode('|', $tieredPricingDiscount);
				if(!isset($tieredPricingDiscount[0]) || !isset($tieredPricingDiscount[1])) {
					continue;
				}

				$minQuantity = $tieredPricingDiscount[0];
				$tieredPricingDiscountDiscount = $tieredPricingDiscount[1];

				$tieredPricingDiscountNext = explode( '|', next($tieredPricingDiscountsApplied[$variationId]['discounts']) );
				if(!isset($tieredPricingDiscountNext[0]) || !isset($tieredPricingDiscountNext[1])) {
					$maxQuantity = 999999999;
				} else {
					$maxQuantity = $tieredPricingDiscountNext[0] - 1;
				}

				if($cartItemQuantity >= $minQuantity && $cartItemQuantity <= $maxQuantity) {

					if($tieredPricingDiscountsApplied[$variationId]['type'] == "percentage") {
						$tieredPricingDiscountDiscountHTML = $tieredPricingDiscountDiscount . '%';
						$tieredPricingPrice = $productPrice - ( ($tieredPricingDiscountDiscount / 100) * $productPrice);
					} elseif($tieredPricingDiscountsApplied[$variationId]['type'] == "fixed_total") {
						$tieredPricingDiscountDiscountHTML = wc_price($tieredPricingDiscountDiscount);
						$tieredPricingPrice = $tieredPricingDiscountDiscount;
					} else {
						$tieredPricingDiscountDiscountHTML = wc_price($tieredPricingDiscountDiscount);
						$tieredPricingPrice = $productPrice - $tieredPricingDiscountDiscount;
					}

					if($tieredPricingPrice < 0) {
						$tieredPricingPrice = 0;
					}

					$tieredPriceApplied[$variationId] = array(
						'price' => $tieredPricingPrice,
						'minQuantity' => $minQuantity,
						'maxQuantity' => $maxQuantity,
					);

					$cartItemProduct->set_price( $tieredPricingPrice );
				}
			}		
		}
	}

	public function change_tiered_price_in_cart($price_html, $cart_item, $cart_item_key ) 
	{
		global $tieredPriceApplied, $productDiscountsApplied;

		if(!isset($cart_item['product_id']) || empty($cart_item['product_id'])) {
			return $price_html;
		}

		if(isset($cart_item['variation_id']) && !empty($cart_item['variation_id'])) {
			$product = wc_get_product($cart_item['variation_id']);	
		} else {
			$product = wc_get_product($cart_item['product_id']);	
		}
		
		if(!$product) {
			return $price_html;
		}
		
		$productId = $product->get_id();
		$variationId = $productId;
		if($product->is_type('variation')) {
			$productId = $product->get_parent_id();
		}


		if(!isset($tieredPriceApplied[$variationId])) {
			return $price_html;
		}

		if(is_array($tieredPriceApplied) && array_key_exists($variationId, $tieredPriceApplied)) {
			$price_html = wc_price($tieredPriceApplied[$variationId]['price']);
		}

		if($this->get_option('tieredPricingBadgeEnabled') || $this->get_option('tieredPricingTooltipEnabled')) {

			// we can not use get_price cause it returns 0
			if(isset($productDiscountsApplied[$variationId])) {
				$originalPrice = $productDiscountsApplied[$variationId]['newPrice']; // $product->get_price();
			} else {
				$originalPrice = get_post_meta($variationId, '_regular_price', true); // $product->get_price();	
				
			}
			
			$newPrice = $tieredPriceApplied[$variationId]['price'];
			$discountRate = round( $originalPrice - $newPrice, 2, PHP_ROUND_HALF_DOWN);
			$discountPercentage = round( ( 1 - ($newPrice / $originalPrice ) ) * 100 );
		}

		$currentPage = 'cart';
		if(is_checkout()) {
			$currentPage = 'checkout';
		}

		// Show Badge
		if($this->get_option('tieredPricingBadgeEnabled')) {

			$text = $this->get_option('tieredPricingBadgeText');

			if($currentPage == "cart" && !in_array('cart', $this->get_option('tieredPricingBadgeDisplay')) ) {
				$text = "";
			}

			if($currentPage == "checkout" && !in_array('checkout', $this->get_option('tieredPricingBadgeDisplay')) ) {
				$text = "";
			}

			if(!empty($text)) {

		 		$text = str_replace(
			    	array(
						'{{discountPercentage}}',
						'{{discountRate}}',
						'{{minQuantity}}',
						'{{maxQuantity}}',
						'{{oldPrice}}',
						'{{newPrice}}',
					),
			    	array(
			    		$discountPercentage . '%',
			    		wc_price($discountRate),
						$tieredPriceApplied[$variationId]['minQuantity'],
						$tieredPriceApplied[$variationId]['maxQuantity'],
						wc_price($originalPrice),
						wc_price($newPrice),
			    	),
			    	$text
			    );

		 		$tooltipText = $this->get_option('tieredPricingTooltipText');
			    if(!empty($tooltipText) && $this->get_option('tieredPricingBadgeTooltip') ) {

					$tooltipText = str_replace(
				    	array(
							'{{discountPercentage}}',
							'{{discountRate}}',
							'{{minQuantity}}',
							'{{maxQuantity}}',
							'{{oldPrice}}',
							'{{newPrice}}',
						),
				    	array(
				    		$discountPercentage . '%',
				    		wc_price($discountRate),
							$tieredPriceApplied[$variationId]['minQuantity'],
							$tieredPriceApplied[$variationId]['maxQuantity'],
							wc_price($originalPrice),
							wc_price($newPrice),
				    	),
				    	$tooltipText
				    );

				    $price_html .= '<span class="woocommerce-ultimate-pricing-tooltip" woocommerce-ultimate-pricing-tooltip="' . strip_tags( $tooltipText ) . '">' . '<span class="woocommerce-ultimate-pricing-badge">' . $text . '</span>' . '</span>';
			    } else {
			    	$price_html .= '<span class="woocommerce-ultimate-pricing-badge">' . $text . '</span>';
			    }
				
			}
		}

		// Show Tooltip
		// Woo strips svg etc.. not working
		if($this->get_option('tieredPricingTooltipEnabled') && $this->get_option('tieredPricingTooltipIconEnabled')) {

			$text = $this->get_option('tieredPricingTooltipText');

			if($currentPage == "cart" && !in_array('cart', $this->get_option('tieredPricingTooltipDisplay')) ) {
				$text = "";
			}

			if($currentPage == "checkout" && !in_array('checkout', $this->get_option('tieredPricingTooltipDisplay')) ) {
				$text = "";
			}

			$tooltipIcon = $this->get_option('tieredPricingTooltipIcon');
			if(!empty($text) && isset($tooltipIcon['url']) && !empty($tooltipIcon['url'])) {

				$tooltipIconURL = $tooltipIcon['url'];

		 		$text = str_replace(
			    	array(
						'{{discountPercentage}}',
						'{{discountRate}}',
						'{{minQuantity}}',
						'{{maxQuantity}}',
						'{{oldPrice}}',
						'{{newPrice}}',
					),
			    	array(
			    		$discountPercentage . '%',
			    		wc_price($discountRate),
						$tieredPriceApplied[$variationId]['minQuantity'],
						$tieredPriceApplied[$variationId]['maxQuantity'],
						wc_price($originalPrice),
						wc_price($newPrice),
			    	),
			    	$text
			    );

				$price_html .= '<span class="woocommerce-ultimate-pricing-tooltip" woocommerce-ultimate-pricing-tooltip="' . strip_tags( $text ) . '"><img width="' . $this->get_option('tieredPricingTooltipIconWidth') . 'px" class="woocommerce-ultimate-pricing-tooltip-icon" src="' . $tooltipIconURL . '"></span>';
			}
		}

		return $price_html;
	}
}